# Copyright dr. ir. R.J.F. Bijster, 2022

from SimulationConstants import *
from SimulationObjects import *

from matplotlib import pyplot as plt

def Run():

    time = 0
    timestep = 0.1 # s
    distance = 11 # km

    route = Trail(
        length_km = distance, 
        numlegs = 17, 
        windspeed = 10/3.6,         # 10 km/h wind head on https://www.knmi.nl/klimaat-viewer/kaarten/wind/gemiddelde-windsnelheid/jaar/Periode_1991-2020
        inclination = 0,
        timestep = timestep
        )

    citybike = Bicycle(
        ridermass = m_rider,
        bikemass = m_citybike,
        maxpower = 100,
        timestep = timestep, 
        acceleration= 1
    )

    pedelec = Bicycle(
        ridermass = m_rider,
        bikemass = m_pedelec,
        maxpower = 250,
        timestep = timestep,
        speedlimiter= 25/3.6,        # m/s, speed limited to 25 km/u
        acceleration= pedelec_a_start_max
    )

    positions1 = []
    speeds1 = []
    states1 = []
    positions2 = []
    speeds2 = []
    states2 = []
    times = []

    while (pedelec.state != BicycleState.FINISHED) or (citybike.state != BicycleState.FINISHED):
        citybike.ride(route)
        pedelec.ride(route)

        pos, speed, state = citybike.getStats()
        
        positions1.append(pos/1000)
        speeds1.append(speed*3.6)
        states1.append(state)

        pos, speed, state = pedelec.getStats()
        
        positions2.append(pos/1000)
        speeds2.append(speed*3.6)
        states2.append(state)
        
        times.append(time)
        time += timestep
        route.update()

    time_to_finish_citybike = times[states1.index(BicycleState.FINISHED)]
    time_to_finish_pedelec = times[states2.index(BicycleState.FINISHED)]
    advantage = time_to_finish_pedelec - time_to_finish_citybike

    # ===== For when you run this simulation for a single run, these print statements and plots will give you
    # some information on the performance. 
    #
    # print(f'citybike finished in {time_to_finish_citybike/60:.1f} min with an average speed = {distance*1000/time_to_finish_citybike*3.6:.1f} km/h')
    # print(f'citybike could have finished in {distance*1000/citybike.maxspeed/60:.1f} min at an average speed of {citybike.maxspeed*3.6:.1f} km/h')
    # print(f'pedelec finished in {time_to_finish_pedelec/60:.1f} min with an average speed = {distance*1000/time_to_finish_pedelec*3.6:.1f} km/h')
    # print(f'pedelec could have finished in {distance*1000/pedelec.maxspeed/60:.1f} min at an average speed of {pedelec.maxspeed*3.6:.1f} km/h')
    # print(f'The time advantage is {advantage:.1f} s or {advantage/60:.1f} min')

    # ##### Plot
    # f, (ax1, ax2, ax3) = plt.subplots(3, 1, sharex=True)
    
    # # Plot the locations of the traffic lights
    # for wp in route.getWayPoints():
    #     ax1.axhline(y=wp.location/1000, color=[0.9, 0.9, 0.9])

    # ax1.plot(times, positions1, times, positions2)

    # ax2.plot(times, speeds1, times, speeds2)
    # ax3.plot(times, states1, label='City bike')
    # ax3.plot(times, states2, label='Pedelec')

    # ax3.set_xlabel('Time (s)')
    # ax1.set_ylabel('Position (km)')
    # ax2.set_ylabel('Speed (km/h)')
    # ax3.set_ylabel('State')

    # ax1.set_ylim([0, distance])

    # max_speed_reached = max(max(speeds1), max(speeds2))
    # ax2.set_ylim([0, max_speed_reached])
    # ax2.set_yticks([0, 10, 20, max_speed_reached])

    # ax3.set_yticks([0, 1, 2, 3, 4])
    # ax3.set_ylim([0, 4])
    # ax3.set_yticklabels(['STOP', 'ACC.', 'CRUISE', 'DEC.', 'FIN.'])
    # ax3.legend()

    # ax3.set_xlim([0, max(times)])
    # plt.show()

    return advantage/60, time_to_finish_citybike/60, time_to_finish_pedelec/60

if __name__ == '__main__':
    Run()