# The following constants are used for comparison
# Note that these are all rough estimates
#
# Copyright dr. ir. R.J.F. Bijster, 2022


# Cyclist
m_male = 84.5                   # kg, average mass dutch male https://opendata.cbs.nl/statline/#/CBS/nl/dataset/81565NED/table?dl=8566
m_female = 71.3                 # kg, average mass dutch female https://opendata.cbs.nl/statline/#/CBS/nl/dataset/81565NED/table?dl=8566
m_rider = 1/2*(m_male + m_female) # kg, average cyclist mass
Pout_cyclist = 75              # W, long duration power output for fit humans, https://www.princeton.edu/~maelabs/hpt/pro/human_3.htm
                                # https://www.fietsersbond.nl/snelheid-5-verschillende-type-fietsen-vergeleken/

# Bicycles
m_citybike = 18                 # kg, https://www.union.nl/blog/gewicht-fiets
m_racingbike = 9                # kg, https://www.union.nl/blog/gewicht-fiets
m_pedelec = 26                  # kg, https://www.union.nl/blog/gewicht-fiets
cr = 0.010                      # -, rolling resistance coefficient, estimated for a city bike

# Wind
ApCd = 0.28                     # m2, Product of projected frontal area and drag coefficient for an upright rider (Debraux 2011)

# Performance
bicycle_a_start_min = 0.8       # m/s^2, Comfortable acceleration rate for normal bicycle, lower bound (CROW 2006)
bicycle_a_start_max = 1.2       # m/s^2, Comfortable acceleration rate for normal bicycle, upper bound (CROW 2006)
bicycle_a_stop_min = 1.5        # m/s^2, Comfortable deceleration rate for normal bicycle, lower bound (CROW 2006)
bicycle_a_stop_max = 2.6        # m/s^2, Emergency deceleration rate for normal bicycle (CROW 2006)

pedelec_a_start_min = 0.6       # m/s^2, Slow acceleration of heavy pedelec (based on Dukulis 2013)
pedelec_a_start_max = 1.6       # m/s^2, Fast acceleration of heavy pedelec (based on Dukulis 2013)
pedelec_a_stop_min = 1.5        # m/s^2, Comfortable deceleration rate for normal bicycle, lower bound (CROW 2006) - No reason to assume it is different for ebikes
pedelec_a_stop_max = 2.6        # m/s^2, Emergency deceleration rate for normal bicycle (CROW 2006) - No reason to assume it is different for ebikes

# Traffic lights
red_phase_max_law = 120         # s, dutch law says the maximum waiting time at a traffic light may not exceed 2 minutes
red_phase_max_wait = 80         # s, https://www.crow.nl/downloads/pdf/verkeer-en-vervoer/verkeersmanagement/basis-voor-een-nota-verkeerslichten-versie-1-2_tcm.aspx?ext=.pdf

cycle_time = 90                 # s, Time between green-to-green phase, 90 - 120 s
green_phase = 8                 # s, Duration of green-phase https://www.wegenwiki.nl/Groenfase
orange_phase = 3                # s, Duration of orange-phase https://www.verkeerskunde.nl/artikel/betere-geeltijden-vk-2-2016

# Physics
g = 9.81        # m/s^2, Gravitational acceleration
rho = 1.225     # kg/m3, Standardized average density of air
p = 101325      # Pa, Standardized average static air pressure

# Short hands ---------------------------------------------------------------------------------------
# v = speed relative to surface
# vw = wind speed
# theta = road inclination

# P_air = 1/2*rho*Cd*Ap*(v + vw)^2*v
#       = c1*(v + vw^2)*v 
#       ~ c1*v^3 (no wind)

# P_road = cr * m * g * v
#        = c2 * v

# P_hill = m * g * sin(theta)*v